@@ -168,8 +168,8 @@ def finishOpenID(request):
             {'error': 'OpenID authentication failed.'},
 
             consumer.SUCCESS:
-            {'url': response.identity_url,
-             'sreg': sreg_response.items(),
+            {'url': response.getDisplayIdentifier(),
+             'sreg': sreg_response and sreg_response.items(),
              'pape': pape_response},
             }
 
