@@ -51,6 +51,8 @@ __all__ = [
     'SRegResponse',
     'data_fields',
     'ns_uri',
+    'ns_uri_1_0',
+    'ns_uri_1_1',
     'supportsSReg',
     ]
 
@@ -457,6 +459,9 @@ class SRegResponse(Extension):
             args = success_response.getSignedNS(self.ns_uri)
         else:
             args = success_response.message.getArgs(self.ns_uri)
+        
+        if not args:
+            return None
 
         for field_name in data_fields:
             if field_name in args:
