@@ -121,12 +121,13 @@ from copy import deepcopy
 
 from openid import cryptutil
 from openid import oidutil
+from openid import kvform
 from openid.dh import DiffieHellman
 from openid.store.nonce import mkNonce
 from openid.server.trustroot import TrustRoot, verifyReturnTo
 from openid.association import Association, default_negotiator, getSecretSize
-from openid.message import Message, OPENID_NS, OPENID1_NS, \
-     OPENID2_NS, IDENTIFIER_SELECT, OPENID1_URL_LIMIT
+from openid.message import Message, InvalidOpenIDNamespace, \
+     OPENID_NS, OPENID2_NS, IDENTIFIER_SELECT, OPENID1_URL_LIMIT
 
 HTTP_OK = 200
 HTTP_REDIRECT = 302
@@ -221,7 +222,6 @@ class CheckAuthRequest(OpenIDRequest):
 
     fromMessage = classmethod(fromMessage)
 
-
     def answer(self, signatory):
         """Respond to this request.
 
@@ -417,7 +417,7 @@ class AssociateRequest(OpenIDRequest):
         @returntype: L{AssociateRequest}
         """
         if message.isOpenID1():
-            session_type = message.getArg(OPENID1_NS, 'session_type')
+            session_type = message.getArg(OPENID_NS, 'session_type')
             if session_type == 'no-encryption':
                 oidutil.log('Received OpenID 1 request with a no-encryption '
                             'assocaition session type. Continuing anyway.')
@@ -471,7 +471,11 @@ class AssociateRequest(OpenIDRequest):
             })
         response.fields.updateArgs(OPENID_NS,
                                    self.session.answer(assoc.secret))
-        if self.session.session_type != 'no-encryption':
+
+        if not (self.session.session_type == 'no-encryption' and
+                self.message.isOpenID1()):
+            # The session type "no-encryption" did not have a name
+            # in OpenID v1, it was just omitted.
             response.fields.setArg(
                 OPENID_NS, 'session_type', self.session.session_type)
 
@@ -542,7 +546,6 @@ class CheckIDRequest(OpenIDRequest):
 
         @raises MalformedReturnURL: When the C{return_to} URL is not a URL.
         """
-        self.namespace = OPENID2_NS
         self.assoc_handle = assoc_handle
         self.identity = identity
         self.claimed_id = identity
@@ -562,7 +565,15 @@ class CheckIDRequest(OpenIDRequest):
             raise MalformedReturnURL(None, self.return_to)
         if not self.trustRootValid():
             raise UntrustedReturnURL(None, self.return_to, self.trust_root)
+        self.message = None
 
+    def _getNamespace(self):
+        warnings.warn('The "namespace" attribute of CheckIDRequest objects '
+                      'is deprecated. Use "message.getOpenIDNamespace()" '
+                      'instead', DeprecationWarning, stacklevel=2)
+        return self.message.getOpenIDNamespace()
+
+    namespace = property(_getNamespace)
 
     def fromMessage(klass, message, op_endpoint):
         """Construct me from an OpenID message.
@@ -586,7 +597,6 @@ class CheckIDRequest(OpenIDRequest):
         """
         self = klass.__new__(klass)
         self.message = message
-        self.namespace = message.getOpenIDNamespace()
         self.op_endpoint = op_endpoint
         mode = message.getArg(OPENID_NS, 'mode')
         if mode == "checkid_immediate":
@@ -597,35 +607,42 @@ class CheckIDRequest(OpenIDRequest):
             self.mode = "checkid_setup"
 
         self.return_to = message.getArg(OPENID_NS, 'return_to')
-        if self.namespace == OPENID1_NS and not self.return_to:
+        if message.isOpenID1() and not self.return_to:
             fmt = "Missing required field 'return_to' from %r"
             raise ProtocolError(message, text=fmt % (message,))
 
         self.identity = message.getArg(OPENID_NS, 'identity')
-        if self.identity and message.isOpenID2():
-            self.claimed_id = message.getArg(OPENID_NS, 'claimed_id')
-            if not self.claimed_id:
+        self.claimed_id = message.getArg(OPENID_NS, 'claimed_id')
+        if message.isOpenID1():
+            if self.identity is None:
+                s = "OpenID 1 message did not contain openid.identity"
+                raise ProtocolError(message, text=s)
+        else:
+            if self.identity and not self.claimed_id:
                 s = ("OpenID 2.0 message contained openid.identity but not "
                      "claimed_id")
                 raise ProtocolError(message, text=s)
-
-        else:
-            self.claimed_id = None
-
-        if self.identity is None and self.namespace == OPENID1_NS:
-            s = "OpenID 1 message did not contain openid.identity"
-            raise ProtocolError(message, text=s)
+            elif self.claimed_id and not self.identity:
+                s = ("OpenID 2.0 message contained openid.claimed_id but not "
+                     "identity")
+                raise ProtocolError(message, text=s)
 
         # There's a case for making self.trust_root be a TrustRoot
         # here.  But if TrustRoot isn't currently part of the "public" API,
         # I'm not sure it's worth doing.
-        if self.namespace == OPENID1_NS:
-            self.trust_root = message.getArg(
-                OPENID_NS, 'trust_root', self.return_to)
-        else:
-            self.trust_root = message.getArg(
-                OPENID_NS, 'realm', self.return_to)
 
+        if message.isOpenID1():
+            trust_root_param = 'trust_root'
+        else:
+            trust_root_param = 'realm'
+
+        # Using 'or' here is slightly different than sending a default
+        # argument to getArg, as it will treat no value and an empty
+        # string as equivalent.
+        self.trust_root = (message.getArg(OPENID_NS, trust_root_param)
+                           or self.return_to)
+
+        if not message.isOpenID1():
             if self.return_to is self.trust_root is None:
                 raise ProtocolError(message, "openid.realm required when " +
                                     "openid.return_to absent")
@@ -671,7 +688,7 @@ class CheckIDRequest(OpenIDRequest):
             return True
         tr = TrustRoot.parse(self.trust_root)
         if tr is None:
-            raise MalformedTrustRoot(None, self.trust_root)
+            raise MalformedTrustRoot(self.message, self.trust_root)
 
         if self.return_to is not None:
             return tr.validateURL(self.return_to)
@@ -683,7 +700,7 @@ class CheckIDRequest(OpenIDRequest):
         response under the realm? It is up to the provider to set a
         policy for what kinds of realms should be allowed. This
         return_to URL verification reduces vulnerability to data-theft
-        attacks based on open proxies, corss-site-scripting, or open
+        attacks based on open proxies, cross-site-scripting, or open
         redirectors.
 
         This check should only be performed after making sure that the
@@ -695,12 +712,16 @@ class CheckIDRequest(OpenIDRequest):
             URL does not support Yadis discovery (and so does not
             support the verification process).
 
+        @raises openid.fetchers.HTTPFetchingError: if the realm URL
+            is not reachable.  When this is the case, the RP may be hosted
+            on the user's intranet.
+
         @returntype: bool
 
         @returns: True if the realm publishes a document with the
             return_to URL listed
 
-        @since: 2.0.2
+        @since: 2.1.0
         """
         return verifyReturnTo(self.trust_root, self.return_to)
 
@@ -746,13 +767,16 @@ class CheckIDRequest(OpenIDRequest):
         @returntype: L{OpenIDResponse}
 
         @change: Version 2.0 deprecates C{server_url} and adds C{claimed_id}.
+
+        @raises NoReturnError: when I do not have a return_to.
         """
-        # FIXME: undocumented exceptions
+        assert self.message is not None
+
         if not self.return_to:
             raise NoReturnToError
 
         if not server_url:
-            if self.namespace != OPENID1_NS and not self.op_endpoint:
+            if not self.message.isOpenID1() and not self.op_endpoint:
                 # In other words, that warning I raised in Server.__init__?
                 # You should pay attention to it now.
                 raise RuntimeError("%s should be constructed with op_endpoint "
@@ -762,7 +786,7 @@ class CheckIDRequest(OpenIDRequest):
 
         if allow:
             mode = 'id_res'
-        elif self.namespace == OPENID1_NS:
+        elif self.message.isOpenID1():
              if self.immediate:
                  mode = 'id_res'
              else:
@@ -775,9 +799,10 @@ class CheckIDRequest(OpenIDRequest):
 
         response = OpenIDResponse(self)
 
-        if claimed_id and self.namespace == OPENID1_NS:
+        if claimed_id and self.message.isOpenID1():
+            namespace = self.message.getOpenIDNamespace()
             raise VersionError("claimed_id is new in OpenID 2.0 and not "
-                               "available for %s" % (self.namespace,))
+                               "available for %s" % (namespace,))
 
         if identity and not claimed_id:
             claimed_id = identity
@@ -806,7 +831,7 @@ class CheckIDRequest(OpenIDRequest):
                         "supplied %r" % (identity,))
                 response_identity = None
 
-            if self.namespace == OPENID1_NS and response_identity is None:
+            if self.message.isOpenID1() and response_identity is None:
                 raise ValueError(
                     "Request was an OpenID 1 request, so response must "
                     "include an identifier."
@@ -824,13 +849,13 @@ class CheckIDRequest(OpenIDRequest):
             if response_identity is not None:
                 response.fields.setArg(
                     OPENID_NS, 'identity', response_identity)
-                if self.namespace == OPENID2_NS:
+                if self.message.isOpenID2():
                     response.fields.setArg(
                         OPENID_NS, 'claimed_id', response_claimed_id)
         else:
             response.fields.setArg(OPENID_NS, 'mode', mode)
             if self.immediate:
-                if self.namespace == OPENID1_NS and not server_url:
+                if self.message.isOpenID1() and not server_url:
                     raise ValueError("setup_url is required for allow=False "
                                      "in OpenID 1.x immediate mode.")
                 # Make a new request just like me, but with immediate=False.
@@ -838,6 +863,10 @@ class CheckIDRequest(OpenIDRequest):
                     self.identity, self.return_to, self.trust_root,
                     immediate=False, assoc_handle=self.assoc_handle,
                     op_endpoint=self.op_endpoint)
+
+                # XXX: This API is weird.
+                setup_request.message = self.message
+
                 setup_url = setup_request.encodeToURL(server_url)
                 response.fields.setArg(OPENID_NS, 'user_setup_url', setup_url)
 
@@ -851,6 +880,8 @@ class CheckIDRequest(OpenIDRequest):
         @type server_url: str
 
         @returntype: str
+
+        @raises NoReturnError: when I do not have a return_to.
         """
         if not self.return_to:
             raise NoReturnToError
@@ -864,15 +895,15 @@ class CheckIDRequest(OpenIDRequest):
              'claimed_id': self.claimed_id,
              'return_to': self.return_to}
         if self.trust_root:
-            if self.namespace == OPENID1_NS:
+            if self.message.isOpenID1():
                 q['trust_root'] = self.trust_root
             else:
                 q['realm'] = self.trust_root
         if self.assoc_handle:
             q['assoc_handle'] = self.assoc_handle
 
-        response = Message(self.namespace)
-        response.updateArgs(self.namespace, q)
+        response = Message(self.message.getOpenIDNamespace())
+        response.updateArgs(OPENID_NS, q)
         return response.toURL(server_url)
 
 
@@ -888,6 +919,8 @@ class CheckIDRequest(OpenIDRequest):
 
         @returntype: str
         @returns: The return_to URL with openid.mode = cancel.
+
+        @raises NoReturnError: when I do not have a return_to.
         """
         if not self.return_to:
             raise NoReturnToError
@@ -895,12 +928,13 @@ class CheckIDRequest(OpenIDRequest):
         if self.immediate:
             raise ValueError("Cancel is not an appropriate response to "
                              "immediate mode requests.")
-        response = Message(self.namespace)
+
+        response = Message(self.message.getOpenIDNamespace())
         response.setArg(OPENID_NS, 'mode', 'cancel')
         return response.toURL(self.return_to)
 
 
-    def __str__(self):
+    def __repr__(self):
         return '<%s id:%r im:%s tr:%r ah:%r>' % (self.__class__.__name__,
                                                  self.identity,
                                                  self.immediate,
@@ -946,16 +980,32 @@ class OpenIDResponse(object):
             self.fields)
 
 
-    def toFormMarkup(self):
+    def toFormMarkup(self, form_tag_attrs=None):
         """Returns the form markup for this response.
 
+        @param form_tag_attrs: Dictionary of attributes to be added to
+            the form tag. 'accept-charset' and 'enctype' have defaults
+            that can be overridden. If a value is supplied for
+            'action' or 'method', it will be replaced.
+
         @returntype: str
 
-        @since: 2.0.2
+        @since: 2.1.0
         """
-        return self.fields.toFormMarkup(
-            self.fields.getArg(OPENID_NS, 'return_to'))
+        return self.fields.toFormMarkup(self.request.return_to,
+                                        form_tag_attrs=form_tag_attrs)
 
+    def toHTML(self, form_tag_attrs=None):
+        """Returns an HTML document that auto-submits the form markup
+        for this response.
+
+        @returntype: str
+
+        @see: toFormMarkup
+
+        @since: 2.1.?
+        """
+        return oidutil.autoSubmitHTML(self.toFormMarkup(form_tag_attrs))
 
     def renderAsForm(self):
         """Returns True if this response's encoding is
@@ -963,7 +1013,7 @@ class OpenIDResponse(object):
 
         @returntype: bool
 
-        @since: 2.0.2
+        @since: 2.1.0
         """
         return self.whichEncoding() == ENCODE_HTML_FORM
 
@@ -983,7 +1033,7 @@ class OpenIDResponse(object):
 
         @returns: one of ENCODE_URL, ENCODE_HTML_FORM, or ENCODE_KVFORM.
 
-        @change: 2.0.2 added the ENCODE_HTML_FORM response.
+        @change: 2.1.0 added the ENCODE_HTML_FORM response.
         """
         if self.request.mode in BROWSER_REQUEST_MODES:
             if self.fields.getOpenIDNamespace() == OPENID2_NS and \
@@ -1167,7 +1217,10 @@ class Signatory(object):
             # dumb mode.
             assoc = self.createAssociation(dumb=True)
 
-        signed_response.fields = assoc.signMessage(signed_response.fields)
+        try:
+            signed_response.fields = assoc.signMessage(signed_response.fields)
+        except kvform.KVFormError, err:
+            raise EncodingError(response, explanation=str(err))
         return signed_response
 
 
@@ -1364,7 +1417,17 @@ class Decoder(object):
         if not query:
             return None
 
-        message = Message.fromPostArgs(query)
+        try:
+            message = Message.fromPostArgs(query)
+        except InvalidOpenIDNamespace, err:
+            # It's useful to have a Message attached to a ProtocolError, so we
+            # override the bad ns value to build a Message out of it.  Kinda
+            # kludgy, since it's made of lies, but the parts that aren't lies
+            # are more useful than a 'None'.
+            query = query.copy()
+            query['openid.ns'] = OPENID2_NS
+            message = Message.fromPostArgs(query)
+            raise ProtocolError(message, str(err))
 
         mode = message.getArg(OPENID_NS, 'mode')
         if not mode:
@@ -1382,7 +1445,7 @@ class Decoder(object):
             L{ProtocolError}.
         """
         mode = message.getArg(OPENID_NS, 'mode')
-        fmt = "No decoder for mode %r"
+        fmt = "Unrecognized OpenID mode %r"
         raise ProtocolError(message, text=fmt % (mode,))
 
 
@@ -1588,7 +1651,7 @@ class ProtocolError(Exception):
         @returntype: str
         """
         if self.openid_message is None:
-            return False
+            return None
         else:
             return self.openid_message.getArg(OPENID_NS, 'return_to')
 
@@ -1627,10 +1690,18 @@ class ProtocolError(Exception):
     def toFormMarkup(self):
         """Encode to HTML form markup for POST.
 
-        @since: 2.0.2
+        @since: 2.1.0
         """
         return self.toMessage().toFormMarkup(self.getReturnTo())
 
+    def toHTML(self):
+        """Encode to a full HTML page, wrapping the form markup in a page
+        that will autosubmit the form.
+
+        @since: 2.1.?
+        """
+        return oidutil.autoSubmitHTML(self.toFormMarkup())
+
     def whichEncoding(self):
         """How should I be encoded?
 
@@ -1691,10 +1762,19 @@ class EncodingError(Exception):
     @type response: L{OpenIDResponse}
     """
 
-    def __init__(self, response):
+    def __init__(self, response, explanation=None):
         Exception.__init__(self, response)
         self.response = response
+        self.explanation = explanation
 
+    def __str__(self):
+        if self.explanation:
+            s = '%s: %s' % (self.__class__.__name__,
+                            self.explanation)
+        else:
+            s = '%s for Response %s' % (
+                self.__class__.__name__, response)
+        return s
 
 
 class AlreadySigned(EncodingError):
