@@ -466,7 +466,7 @@ class TestComplete(TestIdRes):
     def test_idResURLMismatch(self):
         class VerifiedError(Exception): pass
 
-        def discoverAndVerify(_to_match):
+        def discoverAndVerify(claimed_id, _to_match_endpoints):
             raise VerifiedError
 
         self.consumer._discoverAndVerify = discoverAndVerify
@@ -1134,6 +1134,11 @@ class TestReturnToArgs(unittest.TestCase):
             (return_to, {}),
             (return_to + "?another=arg", {(BARE_NS, 'another'): 'arg'}),
             (return_to + "?another=arg#fragment", {(BARE_NS, 'another'): 'arg'}),
+            ("HTTP"+return_to[4:], {}),
+            (return_to.replace('url','URL'), {}),
+            ("http://some.url:80/path", {}),
+            ("http://some.url/p%61th", {}),
+            ("http://some.url/./path", {}),
             ]
 
         endpoint = None
@@ -1171,6 +1176,7 @@ class BadArgCheckingConsumer(GenericConsumer):
         assert args == {
             'openid.mode':'check_authentication',
             'openid.signed':'foo',
+            'openid.ns':OPENID1_NS
             }, args
         return None
 
@@ -1213,7 +1219,6 @@ class TestCheckAuth(unittest.TestCase, CatchLogs):
     def test_signedList(self):
         query = Message.fromOpenIDArgs({
             'mode': 'id_res',
-            'ns': OPENID2_NS,
             'sig': 'rabbits',
             'identity': '=example',
             'assoc_handle': 'munchkins',
@@ -1222,17 +1227,37 @@ class TestCheckAuth(unittest.TestCase, CatchLogs):
             'signed': 'identity,mode,ns.sreg,sreg.email',
             'foo': 'bar',
             })
-        expected = Message.fromOpenIDArgs({
-            'mode': 'check_authentication',
-            'sig': 'rabbits',
-            'assoc_handle': 'munchkins',
-            'identity': '=example',
-            'signed': 'identity,mode,ns.sreg,sreg.email',
-            'ns.sreg': 'urn:sreg',
-            'sreg.email': 'bogus@example.com',
-            })
         args = self.consumer._createCheckAuthRequest(query)
-        self.failUnlessEqual(args.toPostArgs(), expected.toPostArgs())
+        self.failUnless(args.isOpenID1())
+        for signed_arg in query.getArg(OPENID_NS, 'signed').split(','):
+           self.failUnless(args.getAliasedArg(signed_arg), signed_arg)
+
+    def test_112(self):
+        args = {'openid.assoc_handle': 'fa1f5ff0-cde4-11dc-a183-3714bfd55ca8',
+                'openid.claimed_id': 'http://binkley.lan/user/test01',
+                'openid.identity': 'http://test01.binkley.lan/',
+                'openid.mode': 'id_res',
+                'openid.ns': 'http://specs.openid.net/auth/2.0',
+                'openid.ns.pape': 'http://specs.openid.net/extensions/pape/1.0',
+                'openid.op_endpoint': 'http://binkley.lan/server',
+                'openid.pape.auth_policies': 'none',
+                'openid.pape.auth_time': '2008-01-28T20:42:36Z',
+                'openid.pape.nist_auth_level': '0',
+                'openid.response_nonce': '2008-01-28T21:07:04Z99Q=',
+                'openid.return_to': 'http://binkley.lan:8001/process?janrain_nonce=2008-01-28T21%3A07%3A02Z0tMIKx',
+                'openid.sig': 'YJlWH4U6SroB1HoPkmEKx9AyGGg=',
+                'openid.signed': 'assoc_handle,identity,response_nonce,return_to,claimed_id,op_endpoint,pape.auth_time,ns.pape,pape.nist_auth_level,pape.auth_policies'
+                }
+        self.failUnlessEqual(OPENID2_NS, args['openid.ns'])
+        incoming = Message.fromPostArgs(args)
+        self.failUnless(incoming.isOpenID2())
+        car = self.consumer._createCheckAuthRequest(incoming)
+        expected_args = args.copy()
+        expected_args['openid.mode'] = 'check_authentication'
+        expected =Message.fromPostArgs(expected_args)
+        self.failUnless(expected.isOpenID2())
+        self.failUnlessEqual(expected, car)
+        self.failUnlessEqual(expected_args, car.toPostArgs())
 
 
 
@@ -1356,6 +1381,17 @@ class TestSuccessResponse(unittest.TestCase):
         resp = mkSuccess(self.endpoint, {'return_to':'return_to'})
         self.failUnlessEqual(resp.getReturnTo(), 'return_to')
 
+    def test_displayIdentifierClaimedId(self):
+        resp = mkSuccess(self.endpoint, {})
+        self.failUnlessEqual(resp.getDisplayIdentifier(),
+                             resp.endpoint.claimed_id)
+
+    def test_displayIdentifierOverride(self):
+        self.endpoint.display_identifier = "http://input.url/"
+        resp = mkSuccess(self.endpoint, {})
+        self.failUnlessEqual(resp.getDisplayIdentifier(),
+                             "http://input.url/")
+
 class StubConsumer(object):
     def __init__(self):
         self.assoc = object()
@@ -1694,8 +1730,10 @@ class TestDiscoveryVerification(unittest.TestCase):
     def test_otherServer(self):
         text = "verify failed"
 
-        def discoverAndVerify(to_match):
-            self.failUnlessEqual(self.identifier, to_match.claimed_id)
+        def discoverAndVerify(claimed_id, to_match_endpoints):
+            self.failUnlessEqual(claimed_id, self.identifier)
+            for to_match in to_match_endpoints:
+                self.failUnlessEqual(claimed_id, to_match.claimed_id)
             raise ProtocolError(text)
 
         self.consumer._discoverAndVerify = discoverAndVerify
@@ -1719,8 +1757,10 @@ class TestDiscoveryVerification(unittest.TestCase):
     def test_foreignDelegate(self):
         text = "verify failed"
 
-        def discoverAndVerify(to_match):
-            self.failUnlessEqual(self.identifier, to_match.claimed_id)
+        def discoverAndVerify(claimed_id, to_match_endpoints):
+            self.failUnlessEqual(claimed_id, self.identifier)
+            for to_match in to_match_endpoints:
+                self.failUnlessEqual(claimed_id, to_match.claimed_id)
             raise ProtocolError(text)
 
         self.consumer._discoverAndVerify = discoverAndVerify
@@ -1938,7 +1978,9 @@ class TestDiscoverAndVerify(unittest.TestCase):
     def failUnlessDiscoveryFailure(self):
         self.failUnlessRaises(
             DiscoveryFailure,
-            self.consumer._discoverAndVerify, self.to_match)
+            self.consumer._discoverAndVerify,
+            'http://claimed-id.com/',
+            [self.to_match])
 
     def test_noServices(self):
         """Discovery returning no results results in a
@@ -1970,7 +2012,8 @@ class TestDiscoverAndVerify(unittest.TestCase):
 
         # Since _verifyDiscoverySingle returns True, we should get the
         # first endpoint that we passed in as a result.
-        result = self.consumer._discoverAndVerify(self.to_match)
+        result = self.consumer._discoverAndVerify(
+            'http://claimed.id/', [self.to_match])
         self.failUnlessEqual(matching_endpoint, result)
 
 from openid.extension import Extension
